<?php

namespace App\Http\Controllers;

use Validator;
use App\User;
use Firebase\JWT\JWT;
use Illuminate\Http\Request;
use Firebase\JWT\ExpiredException;
use Illuminate\Support\Facades\Hash;
use Laravel\Lumen\Routing\Controller as BaseController;

class AuthController extends BaseController 
{
    /**
     * The request instance.
     *
     * @var \Illuminate\Http\Request
     */
    private $request;

    /**
     * Create a new controller instance.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return void
     */
    public function __construct(Request $request) {
        $this->request = $request;
    }

    /**
     * Create a new token.
     * 
     * @param  \App\User   $user
     * @return string
     */
    protected function jwt(User $user) {
        $payload = [
            'iss' => "lumen-jwt", // Issuer of the token
            'sub' => $user->id, // Subject of the token
            'iat' => time(), // Time when JWT was issued. 
            'exp' => time() + 60*60*24 // Expiration time
        ];
        
        // As you can see we are passing `JWT_SECRET` as the second parameter that will 
        // be used to decode the token in the future.
        return JWT::encode($payload, env('JWT_SECRET'));
    } 

    /**
     * Authenticate a user and return the token if the provided credentials are correct.
     * 
     * @param  \App\User   $user 
     * @return mixed
     */
    public function authenticate(User $user) {
        $this->validate($this->request, [
            'nik'     => 'required',
            'password'  => 'required'
        ]);

        // Find the user by email
        $user = User::where([
                                ['nik','=', $this->request->input('nik')],
                                ['privilege','<>', 'hrd'],
                                ['status','=', 'ACTIVE'],
                            ])->first();

        if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the 
            // below respose for now.
            return response()->json([
                'error' => 'NIK belum terdaftar'
            ]);
        }

        // Verify the password and generate the token
        if (Hash::check($this->request->input('password'), $user->password)) {

            $json = json_decode($user,true);
            $json['token'] = $this->jwt($user);
            return response()->json($json, 200);
        }

        // Bad Request response
        return response()->json([
            'error' => 'Nik atau password salah.'
        ]);
    }

    /**
     * Authenticate a user and return the token if the provided credentials are correct.
     * 
     * @param  \App\User   $user 
     * @return mixed
     */
    public function authenticateHR(User $user) {
        $this->validate($this->request, [
            'nik'     => 'required',
            'password'  => 'required'
        ]);

        // Find the user by email
        $user = User::where('nik', $this->request->input('nik'))
                    ->first();

        if($user->privilege == 'hrd' or
            $user->privilege == 'fin1' or
            $user->privilege == 'fin2' or
            $user->privilege == 'fin3' or
            $user->privilege == 'fin4'){
            
        }
        else{
             return response()->json([
                'error' => 'User tidak bisa memliki hak akses login'
            ]);
        }

        if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the 
            // below respose for now.
            return response()->json([
                'error' => 'Username belum terdaftar'
            ]);
        }

        // Verify the password and generate the token
        if (Hash::check($this->request->input('password'), $user->password)) {

            $json = json_decode($user,true);
            $json['token'] = $this->jwt($user);
            $json['privilege'] = $user->privilege;
            return response()->json($json, 200);
        }

        // Bad Request response
        return response()->json([
            'error' => $user->password
        ]);
    }

    public function authenticateHR_2(User $user) {
        $this->validate($this->request, [
            'nik'     => 'required',
            'password'  => 'required'
        ]);

        // Find the user by email
        $user = User::where('nik', $this->request->input('nik'))
                    ->first();

        if($user->privilege == 'hrd' or
            $user->privilege == 'fin1' or
            $user->privilege == 'fin2' or
            $user->privilege == 'fin3' or
            $user->privilege == 'fin4'){
            
        }
        else{
             return response()->json([
                'error' => 'User tidak bisa memliki hak akses login'
            ]);
        }

        if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the 
            // below respose for now.
            return response()->json([
                'error' => 'Username belum terdaftar'
            ]);
        }

        // Verify the password and generate the token
        if (Hash::check($this->request->input('password'), $user->password)) {
            $json = json_decode($user,true);
            $json['token'] = $this->jwt($user);
            $ret = $json['token'];
            if(!isset($user->privilege)){
                return response()->json([
                    'error' => 'Username tidak ditemukan'
                ]);
            }else{
            //$json['privilege'] = $user->privilege;
        }
            //var_dump($json['token'] );
            return view('changepw',compact('ret'));
            //return response()->json($json, 200);
        }

        // Bad Request response
        return response()->json([
            'error' => 'Error'
        ]);
    }

    /**
     * Authenticate a user and return the token if the provided credentials are correct.
     * 
     * @param  \App\User   $user 
     * @return mixed
     */
    public function changePass(User $user) {
        $this->validate($this->request, [
            'token'         => 'required',
            'oldPassword'   => 'required',
            'newPassword'   => 'required'
        ]);

        $decoded = JWT::decode($this->request->input('token'), env('JWT_SECRET'), array('HS256'));
        $user = User::find($decoded->sub);
        $nik = $user->nik;          


        // Find the user by email
        $user = User::where('nik', $nik)->first();

        if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the 
            // below respose for now.
            return response()->json([
                'error' => 'NIK belum terdaftar'
            ]);
        }

        // Verify the password and generate the token
        if (Hash::check($this->request->input('oldPassword'), $user->password)) {

            $hashNewPass = Hash::make($this->request->input('newPassword'));
            $affected =  User::where('nik',$nik)
                        ->update(['password' => $hashNewPass,'updated_by'=> $nik]);

            if($affected == "1"){
                
                return redirect()->to('http://payroll.padma.co.id');
            }

            return response()->json(['status'=> 'Gagal, ada kesalahan']);
        }

        // Bad Request response
        return response()->json([
            'error' => 'Password lama salah'
        ]);
    }

    /**
     * Authenticate a user and return the token if the provided credentials are correct.
     * 
     * @param  \App\User   $user 
     * @return mixed
     */
    public function forgotPass(User $user) {
        $this->validate($this->request, [
            'nik'     => 'required'
        ]);

        // Find the user by email
        $user = User::where('nik', $this->request->input('nik'))->first();

        if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the 
            // below respose for now.
            return response()->json([
                'error' => 'NIK belum terdaftar'
            ]);
        }

        return response()->json([
                'email' => $user->email
            ]);

    }
}